"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.load = void 0;
const runner_1 = require("../../lib/runner");
const utils_1 = require("../../lib/utils");
function load(commander) {
    commander.program
        .command('ls')
        .description('List not executed migrations')
        .option('--path [path]', 'path to migrations location (default: migrations)')
        .option('--collection [collectionName]', 'database collection name (default: migrations)')
        .option('-E, --executed', 'list only executed')
        .option('-A, --all', 'list all')
        .action(async (options) => {
        const logMigrations = (list) => void console.log(list.join('\n'));
        if (options.all) {
            const list = utils_1.getMigrationsFiles({ migrationsDir: options.path });
            logMigrations(list);
            return;
        }
        if (options.executed) {
            const executed = await runner_1.Runner.instance(options).getExecutedMigrationLogs();
            const foundFiles = utils_1.getMigrationsFiles({ migrationsDir: options.path });
            const list = foundFiles.filter((migrationFileName) => !!executed.filter(([id]) => !!~migrationFileName.indexOf(id)).length);
            logMigrations(list);
            return;
        }
        const executed = await runner_1.Runner.instance(options).getExecutedMigrationLogs();
        const foundFiles = utils_1.getMigrationsFiles({ migrationsDir: options.path });
        const list = foundFiles.filter((migrationFileName) => !executed.filter(([id]) => !!~migrationFileName.indexOf(id)).length);
        logMigrations(list);
    });
}
exports.load = load;
