"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MigrationFileGenerator = void 0;
const fs_1 = __importDefault(require("fs"));
const assert = __importStar(require("assert"));
const utils_1 = require("./utils");
const EMPTY_LINE = '\n';
const EMPTY_STRING = '';
const SPACE_TAB = ' ';
const DEFAULT_TABS_WIDTH = 2;
const SINGLE_QUOTE = `'`;
const DOUBLE_QUOTE = `"`;
const PACKAGE_NAME = require('../package.json').name;
function createStringTabs(tabs) {
    return new Array(tabs).fill(SPACE_TAB).join(EMPTY_STRING);
}
function createMethodTemplate(name, tabWidth) {
    const TABS = createStringTabs(tabWidth);
    const TABSx2 = createStringTabs(tabWidth * 2);
    return [`${TABS}async ${name}(input: IMigrationInput): Promise<void> {`, `${TABSx2}return Promise.resolve(undefined);`, `${TABS}}`];
}
function createIrreversibleTemplate(timestamp, tabWidth, doubleQuote) {
    const QUOTE = doubleQuote ? DOUBLE_QUOTE : SINGLE_QUOTE;
    const HEADER = `import { IIrreversibleMigration, IMigrationInput } from ${QUOTE}${PACKAGE_NAME}/lib/types${QUOTE};${EMPTY_LINE}`;
    const EXECUTE = createMethodTemplate('execute', tabWidth);
    return [HEADER, `export default class IrreversibleMigration${timestamp} implements IIrreversibleMigration {`, ...EXECUTE, '}', EMPTY_STRING].join(EMPTY_LINE);
}
function createReversibleTemplate(timestamp, tabWidth, doubleQuote) {
    const QUOTE = doubleQuote ? DOUBLE_QUOTE : SINGLE_QUOTE;
    const HEADER = `import { IMigrationInput, IReversibleMigration } from ${QUOTE}${PACKAGE_NAME}/lib/types${QUOTE};${EMPTY_LINE}`;
    const UP = createMethodTemplate('up', tabWidth);
    const DOWN = createMethodTemplate('down', tabWidth);
    return [
        HEADER,
        `export default class ReversibleMigration${timestamp} implements IReversibleMigration {`,
        ...UP,
        EMPTY_STRING,
        ...DOWN,
        '}',
        EMPTY_STRING,
    ].join(EMPTY_LINE);
}
// TODO: add proper abstraction to support JS files
exports.MigrationFileGenerator = {
    generate(name, options) {
        const timestamp = Date.now();
        const irreversible = !!options['irreversible'];
        const migrationName = `${timestamp}-${name}.ts`;
        const migrationPath = utils_1.getAbsolutePath({ migrationsDir: options.path, migrationFileName: migrationName });
        const tabs = options.tabs ? Number(options.tabs) : DEFAULT_TABS_WIDTH;
        if (options.clone) {
            const [cloneFileName] = utils_1.getMigrationsFiles({ migrationsDir: options.path, searchString: options.clone });
            assert.ok(cloneFileName, `Filed to find migration file timestamp: ${options.clone}`);
            const clonePath = utils_1.getAbsolutePath({ migrationsDir: options.path, migrationFileName: cloneFileName });
            const cloneFile = fs_1.default
                .readFileSync(clonePath)
                .toString()
                .replace(/(class\s+?)(Reversible|Irreversible)(Migration)([0-9]+)/s, `$1$2$3${timestamp}`);
            fs_1.default.writeFileSync(migrationPath, Buffer.from(cloneFile));
            console.log(`New migration clone created: ${migrationPath}`);
            return;
        }
        if (irreversible) {
            fs_1.default.writeFileSync(migrationPath, Buffer.from(createIrreversibleTemplate(timestamp, tabs, !!options.doubleQuote)));
        }
        else {
            fs_1.default.writeFileSync(migrationPath, Buffer.from(createReversibleTemplate(timestamp, tabs, !!options.doubleQuote)));
        }
        console.log(`New migration file created: ${migrationPath}`);
    },
};
