"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Runner = void 0;
const firebase_admin_1 = __importDefault(require("firebase-admin"));
const types_1 = require("./types");
const utils_1 = require("./utils");
const firestore_1 = require("firebase-admin/lib/firestore");
const assert = __importStar(require("assert"));
var Timestamp = firestore_1.firestore.Timestamp;
class Runner {
    constructor(options) {
        this.app = firebase_admin_1.default.initializeApp();
        this.firestore = this.app.firestore();
        this.migrationCollection = this.firestore.collection(options.collectionName || 'migrations');
        this.migrationsDir = options.path;
    }
    static instance(options) {
        return new Runner(options);
    }
    async migrate(migration) {
        const input = this.createMigrationInput();
        if (utils_1.isIrreversible(migration)) {
            await migration.execute(input);
        }
        else {
            await migration.up(input);
        }
    }
    async rollback(migration) {
        await migration.down(this.createMigrationInput());
    }
    async revert(dryRun, force, searchString) {
        const foundFiles = utils_1.getMigrationsFiles({ migrationsDir: this.migrationsDir, searchString });
        const executedMigrations = await this.getExecutedMigrationLogs();
        const pending = [
            ...(searchString && force
                ? foundFiles
                : foundFiles.filter((migrationFileName) => !!executedMigrations.filter(([id]) => !!~migrationFileName.indexOf(id)).length)),
        ];
        console.info(`Found ${pending.length} migrations to revert.`);
        for (const migrationFile of pending) {
            const [id] = utils_1.getMigrationInfo(migrationFile);
            const migration = await this.createMigration(migrationFile);
            assert.ok(!utils_1.isIrreversible(migration), `Cannot revert ${migrationFile} because it is an irreversible migration.`);
            if (!dryRun) {
                await this.rollback(migration);
                await this.removeMigrationLog(id);
            }
            console.info(`Migration ${migrationFile} reverted.`);
        }
    }
    async run(dryRun, force, searchString) {
        const foundFiles = utils_1.getMigrationsFiles({ migrationsDir: this.migrationsDir, searchString });
        const executedMigrations = await this.getExecutedMigrationLogs();
        const pending = [
            ...(searchString && force
                ? foundFiles
                : foundFiles.filter((migrationFileName) => !executedMigrations.filter(([id]) => !!~migrationFileName.indexOf(id)).length)),
        ];
        console.info(`Found ${pending.length} migrations to run.`);
        for (const migrationFile of pending) {
            const [id, timestamp, name] = utils_1.getMigrationInfo(migrationFile);
            const migration = await this.createMigration(migrationFile);
            if (!dryRun) {
                const type = utils_1.isIrreversible(migration) ? types_1.MigrationType.irreversible : types_1.MigrationType.reversible;
                await this.migrate(migration);
                await this.saveMigrationLog(id, {
                    type,
                    timestamp,
                    name,
                    executedAt: new Date(),
                });
            }
            console.info(`Migration ${migrationFile} executed.`);
        }
    }
    async createMigration(migrationFileName) {
        const migrationPath = utils_1.getAbsolutePath({ migrationsDir: this.migrationsDir, migrationFileName });
        const { default: MigrationClass } = await Promise.resolve().then(() => __importStar(require(migrationPath)));
        return new MigrationClass();
    }
    createMigrationInput() {
        return { firestore: this.firestore, app: this.app };
    }
    async saveMigrationLog(id, fields) {
        await this.migrationCollection.doc(id).set({
            ...fields,
            ...((fields === null || fields === void 0 ? void 0 : fields.executedAt) && { executedAt: Timestamp.fromDate(fields.executedAt) }),
        }, { merge: true });
    }
    async removeMigrationLog(id) {
        await this.migrationCollection.doc(id).delete();
    }
    async getExecutedMigrationLogs() {
        return this.migrationCollection
            .orderBy('timestamp', 'asc')
            .get()
            .then((snapshot) => snapshot.docs.map((doc) => [doc.id, doc.data()]));
    }
}
exports.Runner = Runner;
