"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMigrationInfo = exports.isIrreversible = exports.getMigrationsFiles = exports.getAbsolutePath = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const assert_1 = __importDefault(require("assert"));
const getAbsolutePath = (input) => {
    const absolutePath = input.migrationsDir || 'migrations';
    if (!fs_1.default.existsSync(absolutePath)) {
        fs_1.default.mkdirSync(absolutePath);
    }
    if (input.migrationFileName) {
        return path_1.default.resolve(process.cwd(), absolutePath, input.migrationFileName);
    }
    return path_1.default.resolve(process.cwd(), absolutePath);
};
exports.getAbsolutePath = getAbsolutePath;
const getMigrationsFiles = (input) => {
    const absolutePath = exports.getAbsolutePath({ migrationsDir: input.migrationsDir });
    const files = fs_1.default.readdirSync(absolutePath).filter((fileName) => /[0-9]+-.+\.(ts|js)/.test(fileName));
    if (input.searchString) {
        return files.filter((fileName) => !!~fileName.indexOf(input.searchString));
    }
    return files;
};
exports.getMigrationsFiles = getMigrationsFiles;
const isIrreversible = (migration) => {
    const isObject = (value) => '[object Object]' === Object.prototype.toString.call(migration);
    const isFunction = (value) => ['[object Function]', '[object AsyncFunction]'].includes(Object.prototype.toString.call(value));
    return isObject(migration) && isFunction(migration['execute']);
};
exports.isIrreversible = isIrreversible;
const getMigrationInfo = (fileName, ext = '.ts') => {
    const match = /([0-9]+)-(.+)/s.exec(fileName);
    assert_1.default.ok(match);
    return [path_1.default.basename(fileName, ext), match[1], path_1.default.basename(match[2], '.ts')];
};
exports.getMigrationInfo = getMigrationInfo;
